@@ -5,6 +5,11 @@
 #'   1/n sum_{i = 1}^n Cov[Wi, Yi | X = Xi] / Var[Wi | X = Xi].
 #' Note that for a binary unconfounded treatment, the
 #' average partial effect matches the average treatment effect.
+#' 
+#' If clusters are specified, then each cluster gets equal weight. For example,
+#' if there are 10 clusters with 1 unit each and per-cluster APE = 1, and there
+#' are 10 clusters with 19 units each and per-cluster APE = 0, then the overall
+#' APE is 0.5 (not 0.05).
 #'
 #' @param forest The trained forest.
 #' @param calibrate.weights Whether to force debiasing weights to match expected
@@ -13,6 +18,7 @@
 #'               estimate the ATE. WARNING: For valid statistical performance,
 #'               the subset should be defined only using features Xi, not using
 #'               the treatment Wi or the outcome Yi.
+#' @param num.trees.for.variance Number of trees used to estimate Var[Wi | Xi = x].
 #'
 #' @examples \dontrun{
 #' n = 2000; p = 10
@@ -27,7 +33,10 @@
 #'
 #' @return An estimate of the average partial effect, along with standard error.
 #' @export
-average_partial_effect = function(forest, calibrate.weights = TRUE, subset = NULL) {
+average_partial_effect = function(forest,
+                                  calibrate.weights = TRUE,
+                                  subset = NULL,
+                                  num.trees.for.variance = 500) {
   
   if (!("causal_forest" %in% class(forest))) {
     stop("Average effect estimation only implemented for causal_forest")
@@ -45,37 +54,48 @@ average_partial_effect = function(forest, calibrate.weights = TRUE, subset = NUL
     stop(paste("If specified, subset must be a vector contained in 1:n,",
                "or a boolean vector of length n."))
   }
+  
+  cluster.se <- length(forest$clusters) > 0
+  if (!cluster.se) {
+    clusters <- 1:length(forest$Y.hat)
+    observation.weight <- rep(1, length(forest$Y.hat))
+  } else {
+    clusters <- forest$clusters
+    clust.factor <- factor(clusters)
+    inverse.counts <- 1/as.numeric(Matrix::colSums(Matrix::sparse.model.matrix(~ clust.factor + 0)))
+    observation.weight <- inverse.counts[as.numeric(clust.factor)]
+  }
 
   # Only use data selected via subsetting.
+  subset.X.orig <- forest$X.orig[subset, , drop = FALSE]
   subset.W.orig <- forest$W.orig[subset]
   subset.W.hat <- forest$W.hat[subset]
   subset.Y.orig <- forest$Y.orig[subset]
   subset.Y.hat <- forest$Y.hat[subset]
   tau.hat <- predict(forest)$predictions[subset]
-  if (length(forest$clusters) == 0) {
-    subset.clusters <- numeric(0)
-  } else {
-    subset.clusters <- forest$clusters[subset]
-  }
+  subset.clusters <- clusters[subset]
+  subset.weights <- observation.weight[subset]
   
   # This is a simple plugin estimate of the APE.
-  cape.plugin <- mean(tau.hat)
+  cape.plugin <- weighted.mean(tau.hat, subset.weights)
   
   # Estimate the variance of W given X. For binary treatments,
   # we get a good implicit estimator V.hat = e.hat (1 - e.hat), and
   # so this step is not needed. Note that if we use the present CAPE estimator
   # with a binary treatment and set V.hat = e.hat (1 - e.hat), then we recover
   # exactly the AIPW estimator of the CATE.
-  variance_forest <- regression_forest(forest$X.orig[subset,],
-                                       (subset.W.orig - subset.W.hat)^2)
+  variance_forest <- regression_forest(subset.X.orig,
+                                       (subset.W.orig - subset.W.hat)^2,
+                                       clusters = subset.clusters,
+                                       num.trees = num.trees.for.variance)
   V.hat <- predict(variance_forest)$predictions
-  debiasing.weights <- (subset.W.orig - subset.W.hat) / V.hat
+  debiasing.weights <- subset.weights * (subset.W.orig - subset.W.hat) / V.hat
   
   # In the population, we want A' %*% weights = b.
   # Modify debiasing weights gamma to make this true, i.e., compute
   # argmin {||gamma - gamma.original||_2^2 : A'gamma = b}
   if (calibrate.weights) {
-    A = cbind(1, subset.W.orig, subset.W.hat) / length(subset.W.orig)
+    A = cbind(1, subset.W.orig, subset.W.hat) / sum(subset.weights)
     b = c(0, 1, 0)
     bias = t(A) %*% debiasing.weights - b
     lambda = solve(t(A) %*% A, bias)
@@ -91,13 +111,13 @@ average_partial_effect = function(forest, calibrate.weights = TRUE, subset = NUL
   
   # Estimate variance using the calibration
   if (length(subset.clusters) == 0) {
-    cape.se = sqrt(mean((debiasing.weights * plugin.residual)^2) / length(subset.W.orig))
+    cape.se = sqrt(mean((debiasing.weights * plugin.residual)^2) / (length(subset.W.orig) - 1))
   } else {
     debiasing.clust = Matrix::sparse.model.matrix(
       ~ factor(subset.clusters) + 0,
       transpose = TRUE) %*% (debiasing.weights * plugin.residual)
-    cape.se = sqrt(sum(debiasing.clust^2) / length(subset.W.orig) /
-                     (length(subset.W.orig) - 1))
+    cape.se = sqrt(sum(debiasing.clust^2) / length(debiasing.clust) /
+                     (length(debiasing.clust) - 1))
   }
   
   return(c(estimate=cape.estimate, std.err=cape.se))
