@@ -21,10 +21,12 @@
 
 #include <cstddef>
 #include <unordered_map>
+#include "Eigen/Dense"
 #include "commons/Data.h"
 #include "prediction/Prediction.h"
 #include "prediction/DefaultPredictionStrategy.h"
 #include "prediction/PredictionValues.h"
+#include "ObjectiveBayesDebiaser.h"
 
 class LocalLinearPredictionStrategy: public DefaultPredictionStrategy {
 
@@ -32,7 +34,7 @@ public:
     LocalLinearPredictionStrategy(const Data *original_data,
                                   const Data *test_data,
                                   std::vector<double> lambdas,
-                                  bool use_unweighted_penalty,
+                                  bool weight_penalty,
                                   std::vector<size_t> linear_correction_variables);
 
     size_t prediction_length();
@@ -43,18 +45,26 @@ public:
     * Lambdas is a set of potential regularization parameters, and the forest will
     *   output predictions along each of these parameters.
     */
-
     std::vector<double> predict(size_t sampleID,
                                 const std::unordered_map<size_t, double>& weights_by_sampleID,
                                 const Observations& observations);
 
+    std::vector<double> compute_variance(
+        size_t sampleID,
+        std::vector<std::vector<size_t>> samples_by_tree,
+        std::unordered_map<size_t, double> weights_by_sampleID,
+        const Observations& observations,
+        uint ci_group_size);
+
 private:
+
     static const std::size_t OUTCOME;
     const Data *original_data;
     const Data *test_data;
     std::vector<double> lambdas;
-    bool use_unweighted_penalty;
+    bool weight_penalty;
     std::vector<size_t> linear_correction_variables;
+    ObjectiveBayesDebiaser bayes_debiaser;
 };
 
 #endif //GRF_LOCALLINEARPREDICTIONSTRATEGY_H
