@@ -20,10 +20,12 @@
 
 #include "RegressionSplittingRule.h"
 
-RegressionSplittingRule::RegressionSplittingRule(Data* data, double alpha) {
-  this->data = data;
-  this->alpha = alpha;
-
+RegressionSplittingRule::RegressionSplittingRule(Data* data,
+                                                 double alpha,
+                                                 double imbalance_penalty):
+    data(data),
+    alpha(alpha),
+    imbalance_penalty(imbalance_penalty) {
   size_t max_num_unique_values = data->get_max_num_unique_values();
   this->counter = new size_t[max_num_unique_values];
   this->sums = new double[max_num_unique_values];
@@ -45,49 +47,51 @@ bool RegressionSplittingRule::find_best_split(size_t node,
                                               std::vector<size_t>& split_vars,
                                               std::vector<double>& split_values) {
 
-  size_t num_samples_node = samples[node].size();
-  size_t min_child_samples = std::max<size_t>(std::ceil(num_samples_node * alpha), 1uL);
+  size_t size_node = samples[node].size();
+  size_t min_child_size = std::max<size_t>(std::ceil(size_node * alpha), 1uL);
 
-  double best_decrease = -1;
-  size_t best_var = 0;
-  double best_value = 0;
-
-  // Compute sum of responses in node
-  double sum_node = 0;
+  // Precompute the sum of outcomes in this node.
+  double sum_node = 0.0;
   for (auto& sample : samples[node]) {
     sum_node += labels_by_sample.at(sample);
   }
 
+  // Initialize the variables to track the best split variable.
+  size_t best_var = 0;
+  double best_value = 0;
+  double best_decrease = -1;
+
   // For all possible split variables
   for (auto& var : possible_split_vars) {
     // Use faster method for both cases
-    double q = (double) num_samples_node / (double) data->get_num_unique_data_values(var);
+    double q = (double) size_node / (double) data->get_num_unique_data_values(var);
     if (q < Q_THRESHOLD) {
-      find_best_split_value_small_q(node, var, sum_node, num_samples_node, min_child_samples,
+      find_best_split_value_small_q(node, var, sum_node, size_node, min_child_size,
                                     best_value, best_var, best_decrease, labels_by_sample, samples);
     } else {
-      find_best_split_value_large_q(node, var, sum_node, num_samples_node, min_child_samples,
+      find_best_split_value_large_q(node, var, sum_node, size_node, min_child_size,
                                     best_value, best_var, best_decrease, labels_by_sample, samples);
     }
   }
 
-// Stop if no good split found
+  // Stop if no good split found
   if (best_decrease < 0) {
     return true;
   }
 
-// Save best values
+  // Save best values
   split_vars[node] = best_var;
   split_values[node] = best_value;
   return false;
 }
 
-void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t var, double sum_node,
-                                                            size_t num_samples_node,
-                                                            size_t min_child_samples,
+void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t var,
+                                                            double sum_node,
+                                                            size_t size_node,
+                                                            size_t min_child_size,
                                                             double& best_value, size_t& best_var,
                                                             double& best_decrease,
-                                                            const std::unordered_map<size_t, double>& responses_by_sample,
+                                                            const std::unordered_map<size_t, double>& labels_by_sample,
                                                             const std::vector<std::vector<size_t>>& samples) {
   std::vector<double> possible_split_values;
   data->get_all_values(possible_split_values, samples.at(node), var);
@@ -112,7 +116,7 @@ void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t
   // Sum in right child and possible split
   for (auto& sample : samples[node]) {
     double value = data->get(sample, var);
-    double response = responses_by_sample.at(sample);
+    double response = labels_by_sample.at(sample);
 
     // Count samples until split_value reached
     for (size_t i = 0; i < num_splits; ++i) {
@@ -129,15 +133,25 @@ void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t
   for (size_t i = 0; i < num_splits; ++i) {
 
     // Skip this split if one child is too small.
-    size_t n_left = num_samples_node - n_right[i];
-    if (n_left < min_child_samples || n_right[i] < min_child_samples) {
+    size_t n_left = size_node - n_right[i];
+    if (n_left < min_child_size) {
       continue;
     }
 
+    // Stop if the right child is too small.
+    if (n_right[i] < min_child_size) {
+      break;
+    }
+
     double sum_right = sums_right[i];
     double sum_left = sum_node - sum_right;
     double decrease = sum_left * sum_left / (double) n_left + sum_right * sum_right / (double) n_right[i];
 
+    // Penalize splits that are too close to the edges of the data.
+    double penalty = imbalance_penalty * (1.0 / n_left + 1.0 / n_right[i]);
+    decrease -= penalty;
+
+
     // If better than before, use this
     if (decrease > best_decrease) {
       best_value = possible_split_values[i];
@@ -147,15 +161,16 @@ void RegressionSplittingRule::find_best_split_value_small_q(size_t node, size_t
   }
 }
 
-void RegressionSplittingRule::find_best_split_value_large_q(size_t node, size_t var, double sum_node,
-                                                            size_t num_samples_node,
-                                                            size_t min_child_samples,
+void RegressionSplittingRule::find_best_split_value_large_q(size_t node,
+                                                            size_t var,
+                                                            double sum_node,
+                                                            size_t size_node,
+                                                            size_t min_child_size,
                                                             double& best_value,
                                                             size_t& best_var,
                                                             double& best_decrease,
                                                             const std::unordered_map<size_t, double>& responses_by_sample,
                                                             const std::vector<std::vector<size_t>>& samples) {
-
   // Set counters to 0
   size_t num_unique = data->get_num_unique_data_values(var);
   std::fill(counter, counter + num_unique, 0);
@@ -177,19 +192,23 @@ void RegressionSplittingRule::find_best_split_value_large_q(size_t node, size_t
     sum_left += sums[i];
 
     // Skip to the next value if the left child is too small.
-    if (n_left < min_child_samples) {
+    if (n_left < min_child_size) {
       continue;
     }
 
     // Stop if the right child is too small.
-    size_t n_right = num_samples_node - n_left;
-    if (n_right < min_child_samples) {
+    size_t n_right = size_node - n_left;
+    if (n_right < min_child_size) {
       break;
     }
 
     double sum_right = sum_node - sum_left;
     double decrease = sum_left * sum_left / (double) n_left + sum_right * sum_right / (double) n_right;
 
+    // Penalize splits that are too close to the edges of the data.
+    double penalty = imbalance_penalty * (1.0 / n_left + 1.0 / n_right);
+    decrease -= penalty;
+
     // If better than before, use this
     if (decrease > best_decrease) {
       best_value = data->get_unique_data_value(var, i);
